/*+- OmniWorks Replacement History - Hzproducts`GenericComponents`DataBasedAlarms`AlarmsManagerUI:AlarmsManagerUI.cs;2 */
/*       2*[562893] 02-APR-2004 05:04:05 (GMT) sthakur */
/*         "Modify Alarms Manager Code after Integration testing" */
/*       1*[561200] 01-APR-2004 20:18:15 (GMT) rshethia */
/*         "Alarms Manager UI Main class" */
/*+- OmniWorks Replacement History - Hzproducts`GenericComponents`DataBasedAlarms`AlarmsManagerUI:AlarmsManagerUI.cs;2 */
using System;
using System.Drawing;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;

using BulkData;
using ContextInfo;
using Slb.Horizon.Framework.Dsc;
using Slb.Horizon.UI.Products.GenericComponents;
using Slb.Horizon.NewUIFramework;
using Slb.Horizon.UI.Products.Controls;
using EventsInterface;

using UnitServer;
using Syncfusion.Windows.Forms.Grid;
using nsDataExplorerForm;
using Interface;


using AlarmsManagerUIServer;

namespace Slb.Horizon.UI.Products.Controls
{
	/// <summary>
	/// UI to display, Add and Modify Custom Alarms.
	/// </summary>
	public class AlarmsManagerUI : UIImpl
	{

		#region Variables
		// Tables to hold Alarm grid data
		private System.Data.DataTable m_AlarmDetailsTable;

		// Data Explorer Control
		private Slb.Horizon.UI.Products.GenericComponents.SlbDataExplorerUI m_DataExp;
		
		// Form to contain Data Explorer Control
		private DataExplorerForm DataExpForm;

		// Well Log Document
		private WellLogDocument.ISlbWellLogDocument m_WLD;

		// Representation collection object
		private ISlbDscRepresentationCollection		m_RepCollection;
		
		// Unit System object.
		// TODO This will be replaced by UIUnitServer
		private Interface.ISlbUnits		m_UnitServer;

		// Reference to UI Server
		private AlarmsManagerUIServer.ISlbAlarmsManagerUIServer m_AlarmUIServer;
		
		// Store current display unit system
		private string	m_DisplayUnitSystem;
		
		// XML for getting existent Alarm Information from UI Server
		private string	m_ExistentAlarmInfoXml = string.Empty;

		// XML for sending Modified Alarm Information
		private string	m_ModifiedAlarmInfoXml = string.Empty;
		
		// XML for sending Added Alarm Information
		private string	m_AddedAlarmInfoXml = string.Empty;

		// String for update WORKSPACE CHANGED Event
		private const string	m_EVENT_WORKSPACECHANGED = "WorkSpaceChanged";

		// Alarm detail Variable
		private string m_ChannelOSDD = string.Empty;
		private string m_ToolPath = string.Empty;
		private string m_UnitQuantity = string.Empty;
		private string m_Unit = string.Empty;
		private Interface.DataType_t m_DataType = Interface.DataType_t.DATA_TYPE_INT32;
		private string m_DepthUnit = string.Empty;
		private string m_RunGuid = string.Empty;
		
		// Well Log Document
		private int m_WLDType;

		// String Array to store representation from DataExplorer
		private string m_LapGUID;
		private string[] m_SelectedRep;

		// Delay Types
		private StringCollection DelayTypeCollection;

		// Indicates if there are some Alarms that are modified or added
		private bool m_AnyModifiedAlarms = false;
		private bool m_AnyAddedAlarms = false;

		#endregion

		#region Constant
		
		// Constant for Data Table column Index
		// Column Index for grid is Data Table column Index + 1
		private const int MaxColumn = 18;
		private const int DeleteColIndex = 0;
		private const int ChannelCodeColIndex = 1;
		private const int UnitColIndex = 2;
		private const int AlarmOnColIndex = 3;
		private const int BeepOnColIndex = 4;
		private const int DisplayGuageColIndex = 5;
		private const int LowerErrorLimitColIndex = 6;
		private const int LowerWarningLimitColIndex = 7;
		private const int UpperWarningLimitColIndex = 8;
		private const int UpperErrorLimitColIndex = 9;
		private const int DelayTypeColIndex = 10;
		private const int DelayColIndex = 11;
		private const int ToolPathColIndex = 12;
		private const int ChannelRepGUIDColIndex = 13;
		private const int UnitQuantityColIndex = 14;
		private const int LapGUIDColIndex = 15;
		private const int IsAddedColIndex = 16;
		private const int IsModifiedColIndex = 17;

		
		// Offset for control location
		private const int Offset = 3;

		// Const for Representation collection subcribtion name
		private const string RepSubcribeName = "SlbAlarmUI";

		// Depth Unit Quantity
		private const string DepthUnitQuantity = "Length";

		#endregion
	
		#region Designer Variables
		private Syncfusion.Windows.Forms.Grid.GridDataBoundGrid gridAlarmDetails;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Panel pnlTop;
		private System.Windows.Forms.ImageList imageList1;
		private System.Windows.Forms.ToolBar tbAlarmUI;
		private System.Windows.Forms.ToolBarButton btnAdd;
		private System.Windows.Forms.ToolBarButton btnDelete;
		private System.Windows.Forms.ToolBarButton btnAlarmON;
		private System.Windows.Forms.ToolBarButton btnBeepON;
		private System.Windows.Forms.ToolBarButton btnSave;
		#endregion
		
		#region Constructor and Destructor
		public AlarmsManagerUI()
		{
			try
			{
				m_AlarmDetailsTable = new DataTable();

				DelayTypeCollection = new StringCollection();

				InitializeComponent();

				setGridProperties();

			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in frmDatabasedAlarmUI() ctor ");
			}
		}

		public AlarmsManagerUI(string str):this()
		{        
		}

		public AlarmsManagerUI(object obj):this()
		{
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if ( m_RepCollection != null )
				{
					m_RepCollection.UnSubscribe();
				}
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

		#region	Initialization methods
		/// <summary>
		/// It initializes the Alarm UI called from the UIImpl.
		/// </summary>
		protected override void OnInitialize()
		{
			bool result = false;
			try
			{
				// Get the current Display Unit System
				GetDisplayUnitSystem();

				// Get Job Guid
				string JobGUID = base.GetEnvironmentVariable("JobGUID").ToString();

				// Create Unit server
				// TODO replace by UIUnitServer
				m_UnitServer = (Interface.ISlbUnits)new UnitServer.UnitsClass();
				m_UnitServer.Init();

				m_DepthUnit = m_UnitServer.UnitForSystem(DepthUnitQuantity, m_DisplayUnitSystem);
			
				// Attach to UI Server
				result = AttachToUIServer(); 

				base.SubscribeToFwkEventHost( m_EVENT_WORKSPACECHANGED, EventType_t.enum_UpdateNotify);

				// Initialize Well Log Document				
				m_AlarmUIServer.GetWLDInitData(out m_RunGuid, out m_WLDType);
				m_WLD = base.GetWLD(m_RunGuid, (WellLogDocument.WellLogDocumentType_t)m_WLDType) 
					as WellLogDocument.WellLogDocument;
				if( m_WLD == null )
					throw new Exception( "Null WellLogDocument" );

				//m_WLD.Init(RunGuid, (WellLogDocument.WellLogDocumentType_t)WLDType, WellLogDocument.WellLogDocumentSource_t.WELL_LOG_DOCUMENT_SOURCE_HZDATASTORE);

				// Get Representation collection from Well Log Document
				
				m_RepCollection = m_WLD.GetRepresentationCollection();
				m_RepCollection.Subscribe(RepSubcribeName);
				
				if(result)
				{
					InitializeAlarmGrid();
				}
				
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message,"Initialization Error: " + exp.Message,
					MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
		}


		/// <summary>
		/// Create Data Table and link grid to data table.
		/// Set Grid properties
		/// </summary>
		private void setGridProperties()
		{
			try
			{
				// Set Delay type
				DelayTypeCollection.Add("Time");
				DelayTypeCollection.Add("Depth");

				// Add Columns to Datatable
				m_AlarmDetailsTable.Columns.AddRange(
					new DataColumn[]{
										new DataColumn("Delete",typeof(bool)),
										new DataColumn("Channel Code",typeof(string)),
										new DataColumn("Unit",typeof(string)),
										new DataColumn("Alarm",typeof(bool)),
										new DataColumn("Beep",typeof(bool)),
										new DataColumn("Display Data Gauge",typeof(bool)),
										new DataColumn("Lower Error",typeof(double)),
										new DataColumn("Lower Warning",typeof(double)),
										new DataColumn("Upper Warning",typeof(double)),
										new DataColumn("Upper Error",typeof(double)),
										new DataColumn("Delay Type",typeof(string)),
										new DataColumn("Delay",typeof(int)),
										new DataColumn("Tool Path",typeof(string)),
										new DataColumn("Channel Rep",typeof(string)),
										new DataColumn("Unit Quantity",typeof(string)),
										new DataColumn("LapGUID",typeof(string)),
										new DataColumn("IsAdded",typeof(bool)),
										new DataColumn("IsModified",typeof(bool))}
					);
			
				// Alarm Details grid properties
				gridAlarmDetails.DataSource = m_AlarmDetailsTable;
				gridAlarmDetails.AllowResizeToFit = true;
				gridAlarmDetails.ThemesEnabled = true;
				gridAlarmDetails.ExcelLikeSelectionFrame = false;
				gridAlarmDetails.ExcelLikeCurrentCell = false;
				gridAlarmDetails.MinResizeColSize = 50;
				gridAlarmDetails.ExcelLikeSelectionFrame = false;
				gridAlarmDetails.EnableAddNew = false;
				gridAlarmDetails.EnableRemove = false;
				gridAlarmDetails.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.HideAlways;
				gridAlarmDetails.TableStyle.CheckBoxOptions = new Syncfusion.Windows.Forms.Grid.GridCheckBoxCellInfo(true.ToString(), false.ToString(), "", true);
				
				
				// Setting some the column as hidden
				gridAlarmDetails.Model.Cols.Hidden[ToolPathColIndex + 1] = true;
				gridAlarmDetails.Model.Cols.Hidden[ChannelRepGUIDColIndex + 1] = true;
				gridAlarmDetails.Model.Cols.Hidden[UnitQuantityColIndex + 1] = true;
				gridAlarmDetails.Model.Cols.Hidden[LapGUIDColIndex + 1] = true;
				gridAlarmDetails.Model.Cols.Hidden[IsAddedColIndex + 1] = true;
				gridAlarmDetails.Model.Cols.Hidden[IsModifiedColIndex + 1] = true;
				
				// Set Hidden Column Width to zero
				gridAlarmDetails.Model.ColWidths[ToolPathColIndex + 1] = 0;
				gridAlarmDetails.Model.ColWidths[ChannelRepGUIDColIndex + 1] = 0;
				gridAlarmDetails.Model.ColWidths[UnitQuantityColIndex + 1] = 0;
				gridAlarmDetails.Model.ColWidths[LapGUIDColIndex + 1] = 0;
				gridAlarmDetails.Model.ColWidths[IsAddedColIndex + 1] = 0;
				gridAlarmDetails.Model.ColWidths[IsModifiedColIndex + 1] = 0;

				for(int Index = 0; Index < MaxColumn; Index++)
				{
					switch (Index)
					{
							// Checkbox Columns
						case DeleteColIndex :
						case AlarmOnColIndex :
						case BeepOnColIndex :
						case DisplayGuageColIndex :
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.CellType = "CheckBox";
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.CellValueType = typeof(bool);
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.CellValue = false;
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.Clickable = true;
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.Enabled = true;
							break;
					
							// Editable columns with datatype double and precision 2.
						case LowerWarningLimitColIndex :
						case UpperWarningLimitColIndex :
						case LowerErrorLimitColIndex :
						case UpperErrorLimitColIndex :
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.Format = "F2";
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.ReadOnly = false;
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.Enabled = true;
							break;	

							// Editable column with datatype int.
						//case TimeDelayColIndex :
						case DelayColIndex :
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.ReadOnly = false;
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.Enabled = true;
							break;	

						case DelayTypeColIndex:
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.CellType = "ComboBox";
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.ChoiceList = DelayTypeCollection;
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.ExclusiveChoiceList = true;	
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.ReadOnly = false;
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.Enabled = true;
							break;	
					
							// Hidden and other columns
						case ChannelCodeColIndex :
						case ChannelRepGUIDColIndex :
						case UnitColIndex:
						case ToolPathColIndex :
						case UnitQuantityColIndex:
						case LapGUIDColIndex:
						case IsAddedColIndex:
						case IsModifiedColIndex:
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.CellType = "Static";
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.ReadOnly = true;
							gridAlarmDetails.Binder.InternalColumns[Index].StyleInfo.BackColor = Color.OldLace;
							break;	
										
					}	
				}
				m_AlarmDetailsTable.Clear();
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in setGridProperties()");
			}
		}
		#endregion 
		
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(AlarmsManagerUI));
			this.gridAlarmDetails = new Syncfusion.Windows.Forms.Grid.GridDataBoundGrid();
			this.pnlTop = new System.Windows.Forms.Panel();
			this.tbAlarmUI = new System.Windows.Forms.ToolBar();
			this.btnAdd = new System.Windows.Forms.ToolBarButton();
			this.btnDelete = new System.Windows.Forms.ToolBarButton();
			this.btnSave = new System.Windows.Forms.ToolBarButton();
			this.btnAlarmON = new System.Windows.Forms.ToolBarButton();
			this.btnBeepON = new System.Windows.Forms.ToolBarButton();
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			((System.ComponentModel.ISupportInitialize)(this.gridAlarmDetails)).BeginInit();
			this.pnlTop.SuspendLayout();
			this.SuspendLayout();
			// 
			// gridAlarmDetails
			// 
			this.gridAlarmDetails.ActivateCurrentCellBehavior = Syncfusion.Windows.Forms.Grid.GridCellActivateAction.SelectAll;
			this.gridAlarmDetails.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Row | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.AlphaBlend)));
			this.gridAlarmDetails.ForeColor = System.Drawing.SystemColors.ControlText;
			this.gridAlarmDetails.ListBoxSelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
			this.gridAlarmDetails.Location = new System.Drawing.Point(3, 35);
			this.gridAlarmDetails.MinResizeColSize = 3;
			this.gridAlarmDetails.MinResizeRowSize = 17;
			this.gridAlarmDetails.Name = "gridAlarmDetails";
			this.gridAlarmDetails.Properties.RowHeaders = false;
			this.gridAlarmDetails.ResizeColsBehavior = Syncfusion.Windows.Forms.Grid.GridResizeCellsBehavior.None;
			this.gridAlarmDetails.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.GrayWhenLostFocus;
			this.gridAlarmDetails.Size = new System.Drawing.Size(590, 205);
			this.gridAlarmDetails.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.SingleClick;
			this.gridAlarmDetails.TabIndex = 0;
			this.gridAlarmDetails.Text = "gridDataBoundGrid1";
			this.gridAlarmDetails.ThemesEnabled = true;
			this.gridAlarmDetails.Resize += new System.EventHandler(this.gridAlarmDetails_Resize);
			this.gridAlarmDetails.CurrentCellChanged += new System.EventHandler(this.gridAlarmDetails_CurrentCellChanged);
			this.gridAlarmDetails.PrepareViewStyleInfo += new Syncfusion.Windows.Forms.Grid.GridPrepareViewStyleInfoEventHandler(this.gridAlarmDetails_PrepareViewStyleInfo);
			
			MessageBox.Show("new Draging in disable");
			// Donot allow draging
			this.gridAlarmDetails.AllowDragSelectedCols = false;
			this.gridAlarmDetails.Model.Options.AllowDragSelectedRows = false;
			this.gridAlarmDetails.Model.Options.AllowDragSelectedCols = false;
			this.gridAlarmDetails.Model.Options.DragDropDropTargetFlags = 0;
			this.gridAlarmDetails.Model.Options.DragSelectedCellsMouseButtonsMask = System.Windows.Forms.MouseButtons.None;
			// 
			// pnlTop
			// 
			this.pnlTop.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pnlTop.Controls.Add(this.tbAlarmUI);
			this.pnlTop.Location = new System.Drawing.Point(0, 0);
			this.pnlTop.Name = "pnlTop";
			this.pnlTop.Size = new System.Drawing.Size(600, 35);
			this.pnlTop.TabIndex = 7;
			this.pnlTop.Resize += new System.EventHandler(this.pnlTop_Resize);
			// 
			// tbAlarmUI
			// 
			this.tbAlarmUI.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.tbAlarmUI.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																						 this.btnAdd,
																						 this.btnDelete,
																						 this.btnSave,
																						 this.btnAlarmON,
																						 this.btnBeepON});
			this.tbAlarmUI.ButtonSize = new System.Drawing.Size(20, 20);
			this.tbAlarmUI.DropDownArrows = true;
			this.tbAlarmUI.ImageList = this.imageList1;
			this.tbAlarmUI.Location = new System.Drawing.Point(0, 0);
			this.tbAlarmUI.Name = "tbAlarmUI";
			this.tbAlarmUI.ShowToolTips = true;
			this.tbAlarmUI.Size = new System.Drawing.Size(598, 28);
			this.tbAlarmUI.TabIndex = 3;
			this.tbAlarmUI.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.tbAlarmUI_ButtonClick);
			// 
			// btnAdd
			// 
			this.btnAdd.ImageIndex = 4;
			this.btnAdd.ToolTipText = "Add Alarms";
			// 
			// btnDelete
			// 
			this.btnDelete.ImageIndex = 0;
			this.btnDelete.ToolTipText = "Delete Selected Alarms";
			// 
			// btnSave
			// 
			this.btnSave.ImageIndex = 1;
			this.btnSave.ToolTipText = "Save changes";
			// 
			// btnAlarmON
			// 
			this.btnAlarmON.ImageIndex = 2;
			this.btnAlarmON.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.btnAlarmON.ToolTipText = "Alarm ON";
			// 
			// btnBeepON
			// 
			this.btnBeepON.ImageIndex = 3;
			this.btnBeepON.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.btnBeepON.ToolTipText = "Beep ON";
			// 
			// imageList1
			// 
			this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// AlarmsManagerUI
			// 
			this.Controls.Add(this.pnlTop);
			this.Controls.Add(this.gridAlarmDetails);
			this.MinimumHeight = 200;
			this.MinimumWidth = 300;
			this.Name = "AlarmsManagerUI";
			this.Size = new System.Drawing.Size(592, 245);
			this.Resize += new System.EventHandler(this.frmDatabasedAlarmUI_Resize);
			((System.ComponentModel.ISupportInitialize)(this.gridAlarmDetails)).EndInit();
			this.pnlTop.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region Event Handlers
		/// <summary>
		/// Set toolpath as the tooltip for channelosdd code column.
		/// </summary>
		private void gridAlarmDetails_PrepareViewStyleInfo(object sender, Syncfusion.Windows.Forms.Grid.GridPrepareViewStyleInfoEventArgs e)
		{
			try
			{
				// Show tooltip for Channel OSDD code column.
				if ((e.RowIndex > 0)  && (ChannelCodeColIndex == (e.ColIndex - 1)))
				{
					string str = gridAlarmDetails[e.RowIndex,(ToolPathColIndex+1)].CellValue.ToString() ;
					e.Style.CellTipText = str;
				}
			}
			catch(Exception exp)
			{
				MessageBox.Show(" in view " + exp.Message.ToString());
			}

		}

		protected override void OnUpdateNotify(string type , string context, string subscriptionId)
		{
			if( type.Equals( m_EVENT_WORKSPACECHANGED ))
			{
				m_AlarmUIServer.GetWLDInitData(out m_RunGuid, out m_WLDType);

				if( m_WLD != null )
				{
					if( m_RepCollection != null )
						m_RepCollection.UnSubscribe();
				}


				m_WLD = base.GetWLD(m_RunGuid, (WellLogDocument.WellLogDocumentType_t)m_WLDType) 
					as WellLogDocument.WellLogDocument;
				if( m_WLD == null )
					throw new Exception( "Null WellLogDocument" );

				// Get Representation collection from Well Log Document
				m_RepCollection = m_WLD.GetRepresentationCollection();
				m_RepCollection.Subscribe(RepSubcribeName);

				return;
			}

			//	base.OnUpdateNotify( type, context, subscriptionId );
		}

		/// <summary>
		/// This event is raise whenever any of the grid cell is change.
		/// If any of the row data is change set that row as modified.
		/// If AlarmOn or BeepON checkbox in grid is uncheck set corresponding toggle button 
		/// to non-pushed state.
		/// </summary>
		private void gridAlarmDetails_CurrentCellChanged(object sender, System.EventArgs e)
		{
			try
			{
				GridCurrentCell cc = gridAlarmDetails.CurrentCell;

				int ColIndex = cc.ColIndex;
				int RowIndex = cc.RowIndex;
				if ( RowIndex > 0 && ColIndex >= 0)
				{
					switch(ColIndex - 1)
					{
					
							// If AlarmOn Check box is uncheck, set AlarmOn toggle button to non-pushed state
							// Set modified column(Hidden) to true
						case AlarmOnColIndex :
							if(!(bool)gridAlarmDetails[RowIndex, ColIndex].CellValue)
							{
								btnAlarmON.Pushed = false;				
								btnAlarmON.ToolTipText = "Alarm ON";
							}
							gridAlarmDetails[RowIndex, (IsModifiedColIndex+1)].CellValue = true;
							break;

							// If BeepOn Check box is uncheck, set BeepOn toggle button to non-pushed state
							// Set modified column(Hidden) to true
						case BeepOnColIndex :
							if(!(bool)gridAlarmDetails[RowIndex, ColIndex].CellValue)
							{
								btnBeepON.Pushed = false;			
								btnBeepON.ToolTipText = "Beep ON";
							}
							gridAlarmDetails[RowIndex, (IsModifiedColIndex+1)].CellValue = true;
							break;

					
							// Set modified column(Hidden) to true
						case DisplayGuageColIndex :
						case LowerWarningLimitColIndex :
						case UpperWarningLimitColIndex :
						case LowerErrorLimitColIndex :
						case UpperErrorLimitColIndex :
						case DelayColIndex :
						case DelayTypeColIndex :
							gridAlarmDetails[RowIndex, (IsModifiedColIndex+1)].CellValue = true;
							break;	

						

							// Do nothing
							// This condition will never meet as these column are not editable
						case DeleteColIndex :
						case ChannelCodeColIndex :
						case ChannelRepGUIDColIndex :
						case UnitColIndex:
						case ToolPathColIndex :
						case UnitQuantityColIndex:
						case LapGUIDColIndex:
						case IsAddedColIndex:
						case IsModifiedColIndex:
							break;

					}
					gridAlarmDetails.Refresh();
				}
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in gridAlarmDetails_CurrentCellChanged()");
			}
		
		}
		

		/// <summary>
		/// Resize Alarm Detail grid accordingly
		/// </summary>
		private void frmDatabasedAlarmUI_Resize(object sender, System.EventArgs e)
		{
			try
			{
				this.SuspendLayout ();
				gridAlarmDetails.Width = this.Width;
				pnlTop.Width = this.Width;
			
				gridAlarmDetails.Height = this.ClientSize.Height - pnlTop.Height;
				gridAlarmDetails.Location = new Point(0, pnlTop.Height);

				this.ResumeLayout (true);
				this.Refresh();
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in frmDatabasedAlarmUI_Resize()");
			}
		}
		
		/// <summary>
		/// Resize grid accordingly
		/// </summary>
		private void gridAlarmDetails_Resize(object sender, System.EventArgs e)
		{
			setGridColumnWidth();
			gridAlarmDetails.Refresh();
		}

		
		/// <summary>
		/// Call appropriate method depending on the button clicked.
		/// </summary>
		private void tbAlarmUI_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			gridAlarmDetails.CommitChanges();
			// Save Button clicked
			if ( e.Button.Equals( btnSave ) )
			{
				OnSaveClick();
			}
				// Add Button clicked
				// Show Data Explorer to select channels.	
			else if(e.Button.Equals( btnAdd ))
			{
				ShowDataExplorer();
			}
				// Delete Button clicked	
			else if(e.Button.Equals( btnDelete ))
			{
				DeleteSelectedRows();
			}
			else if ( e.Button.Equals( btnAlarmON) )
			{
				setAllAlarmChecked(btnAlarmON.Pushed);
				if(btnAlarmON.Pushed)
					btnAlarmON.ToolTipText = "Alarm OFF";
				else
					btnAlarmON.ToolTipText = "Alarm ON";
			}
			else if(e.Button.Equals( btnBeepON))
			{
				setAllBeepChecked(btnBeepON.Pushed);			
				if(btnBeepON.Pushed)
					btnBeepON.ToolTipText = "Beep OFF";
				else
					btnBeepON.ToolTipText = "Beep ON";
			}	

		}
		

		/// <summary>
		/// Handle OK button clicked on Data Explorer.
		/// 1. Get Selected channels.
		/// 2. Added Selected channels to Alarm grid.
		/// 3. Close Data Explorer Form.
		/// </summary>
		private void m_DataExp_OKClicked(object sender, EventArgs e)
		{
			// Get Selected channels
			m_LapGUID = string.Empty;
			try
			{
				m_DataExp.GetSelectedData(out m_SelectedRep, out m_LapGUID);
			
				// Added Selected channels to Alarm grid.
				AddSelectedChannels();

				// Close Data Explorer Form.
				DataExpForm.Close();
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in m_DataExp_OKClicked()");
			}
		}


		/// <summary>
		/// Handle OK button clicked on Data Explorer.
		/// Do nothing jus close Data Explorer Form.
		/// </summary>
		private void m_DataExp_CancelClicked(object sender, EventArgs e)
		{
			// Close Data Explorer Form.
			DataExpForm.Close();
		}


		/// <summary>
		/// Resize Toggle button Tool bar accordingly.
		/// </summary>
		private void pnlTop_Resize(object sender, System.EventArgs e)
		{
			pnlTop.SuspendLayout();
			
			pnlTop.ResumeLayout(true);
		}

	
		#endregion

		#region Utility Methods
		/// <summary>
		/// Get Channel OSDD code and Tool Path from representatuin collection 
		/// for given representation Index.
		/// </summary>
		/// <param name="info"></param>
		private void GetRepresentationDetails(int repIndex)
		{
			m_ChannelOSDD = string.Empty;
			m_ToolPath = string.Empty;
			m_DataType = Interface.DataType_t.DATA_TYPE_INT32;
			try
			{
				m_ChannelOSDD	= (string)m_RepCollection.get_ItemByIndex(repIndex, RepresentationTokens.OSDDCODE);
				m_ToolPath  = (string)m_RepCollection.get_ItemByIndex(repIndex, RepresentationTokens.TOOLPATH);
				m_DataType = (Interface.DataType_t) m_RepCollection.get_ItemByIndex(repIndex, RepresentationTokens.DATATYPE);
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in GetRepresentationDetails()");
			}
		}


		/// <summary>
		/// Get Unit quantity and Unit for channel.
		/// </summary>
		private void GetUnitDetails(int repIndex, string UnitSystem)
		{
			m_UnitQuantity = string.Empty;
			m_Unit = string.Empty;
			try
			{
				m_UnitQuantity = (string)m_RepCollection.get_ItemByIndex(repIndex, RepresentationTokens.UNITQUANTITY);
				m_Unit = m_UnitServer.UnitForSystem(m_UnitQuantity, UnitSystem);
			
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in GetUnitDetails()");
			}
		}


		/// <summary>
		/// Set Column Width.
		/// </summary>
		private void setGridColumnWidth()
		{
			//gridMessageViewer.MinResizeColSize;
			int totalWidth = 0;
			int tempWidth = 0;
			for (int Index = 1; Index <= (MaxColumn + 1); Index++)
				if (Index != (ChannelCodeColIndex + 1))
					totalWidth += this.gridAlarmDetails.Model.ColWidths[Index];
			tempWidth = this.gridAlarmDetails.ClientSize.Width - totalWidth;;
			if(tempWidth > gridAlarmDetails.Model.ColWidths[ChannelCodeColIndex + 1])
				gridAlarmDetails.Model.ColWidths[ChannelCodeColIndex + 1] = tempWidth;
		}
		

		/// <summary>
		/// Change the check status of all BeepOn Checkbox in grid
		/// </summary>
		private void setAllBeepChecked(bool IsCheck)
		{
			int RowCount = m_AlarmDetailsTable.Rows.Count;
			
			try
			{
				// To lessen flickering
				//gridAlarmDetails.Model.SuspendChangeEvents();
				//gridAlarmDetails.Model.SuspendRecordUndo();	
				gridAlarmDetails.BeginUpdate();
				gridAlarmDetails.Binder.SuspendBinding();

				// For each row in grid
				for(int Index = 0; Index < RowCount; Index++) 
				{
					if(IsCheck != (bool)m_AlarmDetailsTable.Rows[Index][BeepOnColIndex])
					{
						m_AlarmDetailsTable.Rows[Index][BeepOnColIndex] = IsCheck;
						m_AlarmDetailsTable.Rows[Index][IsModifiedColIndex] = true;
					}
				}

				m_AlarmDetailsTable.AcceptChanges();
			
				gridAlarmDetails.Binder.ResumeBinding();
				//gridAlarmDetails.Model.ResumeChangeEvents();
				//gridAlarmDetails.Model.ResumeRecordUndo();
				//gridAlarmDetails.Model.ResetVolatileData();
				gridAlarmDetails.EndUpdate(true);
				gridAlarmDetails.Refresh();
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in setAllBeepChecked()");
			}

		}


		/// <summary>
		/// Change the check status of all BeepOn Checkbox in grid
		/// </summary>
		private void setAllAlarmChecked(bool IsCheck)
		{
			int RowCount = m_AlarmDetailsTable.Rows.Count;
			try
			{
			
				// To lessen flickering
				//gridAlarmDetails.Model.SuspendChangeEvents();
				//gridAlarmDetails.Model.SuspendRecordUndo();	
				gridAlarmDetails.BeginUpdate();
				gridAlarmDetails.Binder.SuspendBinding();

			
				// For each row in grid
				for(int Index = 0; Index < RowCount; Index++) 
				{
				
					if(IsCheck != (bool)m_AlarmDetailsTable.Rows[Index][AlarmOnColIndex])
					{
						m_AlarmDetailsTable.Rows[Index][AlarmOnColIndex] = IsCheck;
						m_AlarmDetailsTable.Rows[Index][IsModifiedColIndex] = true;
					}
				}
				m_AlarmDetailsTable.AcceptChanges();

				gridAlarmDetails.Binder.ResumeBinding();
				//gridAlarmDetails.Model.ResumeChangeEvents();
				//gridAlarmDetails.Model.ResumeRecordUndo();
				//gridAlarmDetails.Model.ResetVolatileData();
				gridAlarmDetails.EndUpdate(true);
				gridAlarmDetails.Refresh();
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in setAllAlarmChecked()");
			}
		}


		/// <summary>
		/// Generate modified Alarm and Added Alarm xml.
		/// Call UIServer method to modify alarms and add new alarms.
		/// </summary>
		private void OnSaveClick()
		{
			m_AnyModifiedAlarms = false;
			m_AnyAddedAlarms = false;
			try
			{
				if(GenerateDataXmlForServer())
				{
					if(m_AnyModifiedAlarms)
						m_AlarmUIServer.UpdateExistentAlarms(m_ModifiedAlarmInfoXml);
					if(m_AnyAddedAlarms)
						m_AlarmUIServer.AddNewAlarms(m_AddedAlarmInfoXml);
				}
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in OnSaveClick()");
			}
		}


		/// <summary>
		/// Generate xml containing channel representation of alarms mark for deletion.
		/// Call UIServer method to delete alarms.
		/// </summary>
		private void DeleteSelectedRows()
		{

			string DeleteAlarmInfoXml = string.Empty;
			int RowCount = m_AlarmDetailsTable.Rows.Count;
			bool IsAlarmForDeletion;
			
			try
			{
				DeleteAlarmInfoXml = "<AlarmCollection>";
				IsAlarmForDeletion = false;

				// For each row in grid
				for(int Index = 0; Index < RowCount; Index++)
				{
					// Check if row is mark for deletion. (Delete check box is check)
					if ((bool)(m_AlarmDetailsTable.Rows[Index][DeleteColIndex]))
					{
						DeleteAlarmInfoXml += "<Alarm><ChannelRepGUID>" + m_AlarmDetailsTable.Rows[Index][ChannelRepGUIDColIndex].ToString() + "</ChannelRepGUID></Alarm>";
						m_AlarmDetailsTable.Rows[Index].Delete();
						IsAlarmForDeletion = true;
					}
				}

				DeleteAlarmInfoXml += "</AlarmCollection>";
			

				if(IsAlarmForDeletion)
				{
					//  Call UI Server funtion to delete selected Alarms
					m_AlarmDetailsTable.AcceptChanges();
					m_AlarmUIServer.DeleteAlarms(DeleteAlarmInfoXml);
				}

				gridAlarmDetails.Refresh();

			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in DeleteSelectedRows()");
			}

		}


		/// <summary>
		/// Show Data Explorer as Modal dialog.
		/// 1. Create Form containing Data Explorer control.
		/// 2. Set Handler for OK and Canel event on Data Explorer.
		/// 3. Init Data Explorer control.
		/// 4. Show Dialog.
		/// </summary>
		private void ShowDataExplorer()
		{
			try
			{
				// Create Form containing Data Explorer control.
				DataExpForm = new DataExplorerForm();

				// Get Data Explorer control object.
				m_DataExp = DataExpForm.m_newDataExplorer;

				// Set Handler for OK and Canel event on Data Explorer.
				m_DataExp.OKClicked += new DataExpEventHandler(m_DataExp_OKClicked);
				m_DataExp.CancelClicked += new DataExpEventHandler(m_DataExp_CancelClicked);
			
				// Init Data Explorer control.
				m_DataExp.Init(m_WLD, m_RunGuid, 
					FocussedItemType.DATA_ACQ_RUN, ReturnItemType.REPRESENTATION, SelectionType.MULTIPLE, 
					ViewStyle.PROPERTIES, DomainType.ALL,m_DisplayUnitSystem, null);
			
				// Show Dialog.
				DataExpForm.ShowDialog();
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in ShowDataExplorer()");
			}
		}


		/// <summary>
		/// Get detail for each channel in array.
		/// Add a row for each channel in Alarm grid.
		/// </summary>
		private void AddSelectedChannels()
		{
			try
			{
				DataRow dtRow;

				// To lessen flickering
//				gridAlarmDetails.Model.SuspendChangeEvents();
//				gridAlarmDetails.Model.SuspendRecordUndo();	
				gridAlarmDetails.BeginUpdate();
				gridAlarmDetails.Binder.SuspendBinding();

				// For each channel rep in array
				for(int Index = 0; Index < m_SelectedRep.Length; Index++)
				{
					// Check if channel is already present in Alarm grid.
					if(!CheckIfAlreadyPresent(m_SelectedRep[Index]))
					{
						m_RepCollection.LockData();
					
						// Get representation Index for given channel representation guid.
						int repIndex = m_RepCollection.get_Index(m_SelectedRep[Index]);

					
						// Get Channel OSDD code and toolpath. 
						GetRepresentationDetails(repIndex);

						// Get unti quantity and unit.
						GetUnitDetails(repIndex, m_DisplayUnitSystem);

						m_RepCollection.UnlockData();


						if(CheckIfDataTypeSupported())
						{
					
							dtRow = m_AlarmDetailsTable.NewRow();
							dtRow[DeleteColIndex] = false;
							dtRow[ChannelCodeColIndex] = m_ChannelOSDD;
							dtRow[UnitColIndex] = m_Unit;
							dtRow[AlarmOnColIndex] = false;
							dtRow[BeepOnColIndex] = false;
							dtRow[DisplayGuageColIndex] = false;
							dtRow[DelayTypeColIndex] = "Time";
							dtRow[DelayColIndex] = 0;
							dtRow[ToolPathColIndex] = m_ToolPath;
							dtRow[ChannelRepGUIDColIndex] = m_SelectedRep[Index];
							dtRow[UnitQuantityColIndex] = m_UnitQuantity;
							dtRow[LapGUIDColIndex] = m_LapGUID;
							dtRow[IsAddedColIndex] = true;
							dtRow[IsModifiedColIndex] = false;

							m_AlarmDetailsTable.Rows.Add(dtRow);	
						}
					}


				}
			

//				gridAlarmDetails.Model.ResumeChangeEvents();
//				gridAlarmDetails.Model.ResumeRecordUndo();
//				gridAlarmDetails.Model.ResetVolatileData();
				gridAlarmDetails.Binder.ResumeBinding();
				gridAlarmDetails.EndUpdate(true);

				// Refresh grid to reflect above changes.
				m_AlarmDetailsTable.AcceptChanges();
				gridAlarmDetails.Refresh();	
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in AddSelectedChannels()");
			}
		}
		

		/// <summary>
		/// Check If Channel is already there in Alarm grid.
		/// </summary>
		private bool CheckIfAlreadyPresent(string ChannelRep)
		{
			int RowCount = m_AlarmDetailsTable.Rows.Count;
			
			// For each row in grid (table)
			for(int Index = 0; Index < RowCount; Index++)
			{
				// If represent return true
				if (ChannelRep.Equals(m_AlarmDetailsTable.Rows[Index][ChannelRepGUIDColIndex].ToString()))
					return true;
			}
		
			return false;
		}


		/// <summary>
		/// Generate data xml to be sent to UIServer for saving unit Alarms 
		/// </summary>
		private bool GenerateDataXmlForServer()
		{
			
			string strAlarmOn = string.Empty;
			string strBeepOn = string.Empty;
			string strDisplayGuage = string.Empty;
			string UnitQuantity = string.Empty;

			string strUpperErrorLimit = string.Empty;
			string strLowerErrorLimit = string.Empty;
			string strUpperWarningLimit = string.Empty;
			string strLowerWarningLimit = string.Empty;

			m_ModifiedAlarmInfoXml = string.Empty;
			m_ModifiedAlarmInfoXml += "<AlarmList>";

			m_AddedAlarmInfoXml = string.Empty;
			m_AddedAlarmInfoXml += "<AlarmList>";

			double tempLimit = 0;
			int tempDeplay = 0;

			try
			{
				// Adding Alarm data ty to the Xml
				for(int Index = 0; Index < m_AlarmDetailsTable.Rows.Count; Index++)
				{
					// Get Unit Quantity
					UnitQuantity = m_AlarmDetailsTable.Rows[Index][UnitQuantityColIndex].ToString();

					// Set the Limits
					strUpperErrorLimit = m_AlarmDetailsTable.Rows[Index][UpperErrorLimitColIndex].ToString();
					strLowerErrorLimit = m_AlarmDetailsTable.Rows[Index][LowerErrorLimitColIndex].ToString();
					strUpperWarningLimit = m_AlarmDetailsTable.Rows[Index][UpperWarningLimitColIndex].ToString();
					strLowerWarningLimit = m_AlarmDetailsTable.Rows[Index][LowerWarningLimitColIndex].ToString();
				
					if(!CheckLimits(strLowerErrorLimit, strLowerWarningLimit, strUpperWarningLimit, strUpperErrorLimit))
					{
						m_ModifiedAlarmInfoXml = string.Empty;
						m_AddedAlarmInfoXml = string.Empty;
						return false;
					}

					if((bool)m_AlarmDetailsTable.Rows[Index][AlarmOnColIndex])
						strAlarmOn = "1";
					else
						strAlarmOn = "0";

					if((bool)m_AlarmDetailsTable.Rows[Index][BeepOnColIndex])
						strBeepOn = "1";
					else
						strBeepOn = "0";

					if((bool)m_AlarmDetailsTable.Rows[Index][DisplayGuageColIndex])
						strDisplayGuage = "1";
					else
						strDisplayGuage = "0";


					if(strUpperErrorLimit.Length > 0)
					{
						tempLimit = double.Parse(strUpperErrorLimit);
						tempLimit = GetValueInCanadianUS(tempLimit, UnitQuantity);
						strUpperErrorLimit = tempLimit.ToString();
					}
					

					if(strLowerErrorLimit.Length > 0)
					{
						tempLimit = double.Parse(strLowerErrorLimit);
						tempLimit = GetValueInCanadianUS(tempLimit, UnitQuantity);
						strLowerErrorLimit = tempLimit.ToString();
					}
				
					if(strUpperWarningLimit.Length > 0)
					{
						tempLimit = double.Parse(strUpperWarningLimit);
						tempLimit = GetValueInCanadianUS(tempLimit, UnitQuantity);
						strUpperWarningLimit = tempLimit.ToString();
					}
				
					if(strLowerWarningLimit.Length > 0)
					{
						tempLimit = double.Parse(strLowerWarningLimit);
						tempLimit = GetValueInCanadianUS(tempLimit, UnitQuantity);
						strLowerWarningLimit = tempLimit.ToString();
					}


					if((bool)m_AlarmDetailsTable.Rows[Index][IsAddedColIndex])
					{
						if(!m_AnyAddedAlarms)
						
						m_AnyAddedAlarms = true;

						m_AddedAlarmInfoXml = m_AddedAlarmInfoXml + "<Alarm>" 
											+ "<ChannelRepGUID>" + m_AlarmDetailsTable.Rows[Index][ChannelRepGUIDColIndex].ToString() + "</ChannelRepGUID>" 
											+ "<ChannelCode>" + m_AlarmDetailsTable.Rows[Index][ChannelCodeColIndex].ToString() + "</ChannelCode>" 
											//+ "<LapGUID>" + m_AlarmDetailsTable.Rows[Index][LapGUIDColIndex].ToString() + "</LapGUID>" 
											+ "<ToolPath>" + m_AlarmDetailsTable.Rows[Index][ToolPathColIndex].ToString() + "</ToolPath>" 

											+ "<UpperErrorLimit>" + strUpperErrorLimit + "</UpperErrorLimit>"
											+ "<LowerErrorLimit>" + strLowerErrorLimit + "</LowerErrorLimit>"
											+ "<UpperWarningLimit>" + strUpperWarningLimit + "</UpperWarningLimit>"
											+ "<LowerWarningLimit>" + strLowerWarningLimit + "</LowerWarningLimit>" ;

						if(m_AlarmDetailsTable.Rows[Index][DelayTypeColIndex].ToString().Equals("Time"))
						{
							if(m_AlarmDetailsTable.Rows[Index][DelayColIndex].ToString().Length > 0)
								m_AddedAlarmInfoXml += "<TimeDelay>" + m_AlarmDetailsTable.Rows[Index][DelayColIndex].ToString() + "</TimeDelay>" ;
							else
								m_AddedAlarmInfoXml += "<TimeDelay>" + "0" + "</TimeDelay>" ;

							m_AddedAlarmInfoXml += "<DepthDelay>" + "" + "</DepthDelay>" ;
						}
						else
						{
							m_AddedAlarmInfoXml += "<TimeDelay>" + "" + "</TimeDelay>" ;

							if(m_AlarmDetailsTable.Rows[Index][DelayColIndex].ToString().Length > 0)
							{
								tempDeplay = int.Parse(m_AlarmDetailsTable.Rows[Index][DelayColIndex].ToString());
								tempDeplay = (int)GetValueInCanadianUS(tempDeplay, DepthUnitQuantity);

								m_AddedAlarmInfoXml += "<DepthDelay>" + tempDeplay.ToString() + "</DepthDelay>" ;
							}
							else
								m_AddedAlarmInfoXml += "<DepthDelay>" + "0" + "</DepthDelay>" ;
						}
					
						m_AddedAlarmInfoXml = m_AddedAlarmInfoXml
												+ "<AlarmOn>" + strAlarmOn + "</AlarmOn>"
												+ "<BeepOn>" + strBeepOn + "</BeepOn>"
												+ "<DisplayGuage>" + strDisplayGuage + "</DisplayGuage>"					
												+ "</Alarm>";

						// Set the Added falg to false
						m_AlarmDetailsTable.Rows[Index][IsAddedColIndex] = false;
						m_AlarmDetailsTable.Rows[Index][IsModifiedColIndex] = false;
					}
					else if((bool)m_AlarmDetailsTable.Rows[Index][IsModifiedColIndex])
					{
						m_AnyModifiedAlarms = true;

						m_ModifiedAlarmInfoXml = m_ModifiedAlarmInfoXml + "<Alarm>" 
												+ "<ChannelRepGUID>" + m_AlarmDetailsTable.Rows[Index][ChannelRepGUIDColIndex].ToString() + "</ChannelRepGUID>"
												+ "<ChannelCode>" + m_AlarmDetailsTable.Rows[Index][ChannelCodeColIndex].ToString() + "</ChannelCode>" 
												//+ "<LapGUID>" + m_AlarmDetailsTable.Rows[Index][LapGUIDColIndex].ToString() + "</LapGUID>" 
												+ "<ToolPath>" + m_AlarmDetailsTable.Rows[Index][ToolPathColIndex].ToString() + "</ToolPath>" 
												+ "<UpperErrorLimit>" + strUpperErrorLimit + "</UpperErrorLimit>" 
												+ "<LowerErrorLimit>" + strLowerErrorLimit + "</LowerErrorLimit>" 
												+ "<UpperWarningLimit>" + strUpperWarningLimit + "</UpperWarningLimit>" 
												+ "<LowerWarningLimit>" + strLowerWarningLimit + "</LowerWarningLimit>" ;
						
						if(m_AlarmDetailsTable.Rows[Index][DelayTypeColIndex].ToString().Equals("Time"))
						{
							m_ModifiedAlarmInfoXml += "<TimeDelay>" + m_AlarmDetailsTable.Rows[Index][DelayColIndex].ToString() + "</TimeDelay>" ;
							m_ModifiedAlarmInfoXml += "<DepthDelay>" + "" + "</DepthDelay>" ;
						}
						else
						{
							m_ModifiedAlarmInfoXml += "<TimeDelay>" + "" + "</TimeDelay>" ;
							m_ModifiedAlarmInfoXml += "<DepthDelay>" + m_AlarmDetailsTable.Rows[Index][DelayColIndex].ToString() + "</DepthDelay>" ;
						}
						m_ModifiedAlarmInfoXml = m_ModifiedAlarmInfoXml
													+ "<AlarmOn>" + strAlarmOn + "</AlarmOn>" 
													+ "<BeepOn>" + strBeepOn + "</BeepOn>" 
													+ "<DisplayGuage>" + strDisplayGuage + "</DisplayGuage>" 
													+ "</Alarm>" ;

						// Set the modified flag to false
						m_AlarmDetailsTable.Rows[Index][IsModifiedColIndex] = false;
					}

				}
				m_AlarmDetailsTable.AcceptChanges();
				gridAlarmDetails.Refresh();

				m_ModifiedAlarmInfoXml += "</AlarmList>";
				m_AddedAlarmInfoXml += "</AlarmList>";
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in GenerateDataXmlForServer()");
				return false;
			}

			return true;

		}


		/// <summary>
		/// Get Current display unit system stored as environment variable.
		/// </summary>
		private void GetDisplayUnitSystem()
		{
			// Get Display Unit System
			//m_DisplayUnitSystem = (string) base.GetEnvironmentVariable("UnitSystem");
			if(m_DisplayUnitSystem == null)
			{
				// Display Unit System is not available.
				// Set it to English by default.
				m_DisplayUnitSystem = "English";
			}


		}


		/// <summary>
		/// Check if the Unit system is custom unit system 
		/// </summary>
		private bool CheckIfCustomUnitSystem(string UnitSystem)
		{
			if((string.Compare(UnitSystem, "ENGLISH", true) == 0)  
				|| (string.Compare(UnitSystem, "CANADIAN", true) == 0)  
				|| (string.Compare(UnitSystem, "METRIC", true) == 0))  
				return false;
			else
				return true;
		}


		/// <summary>
		/// Attach to Alarm UI Server
		/// </summary>
		private bool AttachToUIServer()
		{
			try
			{
				string WorkflowID = (string) GetEnvironmentVariable("WorkflowID");
				m_AlarmUIServer = (ISlbAlarmsManagerUIServer)GetUIServer("AlarmsMgrUIServer.SlbAlarmsMgrUIS", WorkflowID);
				//m_AlarmUIServer.InitializeAlarmsManagerUIS();
			}
			catch 
			{
				MessageBox.Show("Unable to acquire Alarm UIServer.", "Alarm UI", 
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				
				return false;
			}
			return true;		
		}


		/// <summary>
		/// Initialize Alarm grid with existing custom Alarm.
		/// 1. Get the Custom Alarm data from UIServer
		/// 2. Parse the xml.
		/// 3. Populate grid with Alarm data 
		/// </summary>
		private void InitializeAlarmGrid()
		{
			m_ExistentAlarmInfoXml = string.Empty;

			// Get the existent Alarms detail fro UI Server
			m_AlarmUIServer.FetchExistentAlarms(out m_ExistentAlarmInfoXml);

			// If there Existent are Alarms then populate the grid
			if(m_ExistentAlarmInfoXml.Length > 0)
			{
				PopulateAlarmGrid();
				OnDisplayUnitSystemChanged("Canadian", m_DisplayUnitSystem);
			}
		}
		

		/// <summary>
		/// Initialize Alarm grid with existing custom Alarm.
		/// 1. Parse the xml.
		/// 2. Populate grid with Alarm data 
		/// </summary>
		private void PopulateAlarmGrid()
		{
			try
			{
				XmlDocument TmpXmlDoc = new XmlDocument();
				XmlNodeList AlarmNodeList;

				DataRow dtRow;

				string ChannelRepGUID = string.Empty;
				string ChannelCode = string.Empty;
				string ToolPath = string.Empty;
				string LapGUID = string.Empty;

				string DelayType = "Time";
				int Delay = 0;
				bool AlarmOn = false;
				bool BeepOn = false;
				bool DisplayGuage = false;

				TmpXmlDoc.LoadXml(m_ExistentAlarmInfoXml);

				AlarmNodeList = TmpXmlDoc.GetElementsByTagName("Alarm");

				// To lessen flickering
				gridAlarmDetails.Model.SuspendChangeEvents();
				gridAlarmDetails.Model.SuspendRecordUndo();	
				gridAlarmDetails.BeginUpdate();
				gridAlarmDetails.Binder.SuspendBinding();

				string strLowerWarningLimit;
				string strUpperWarningLimit;
				string strLowerErrorLimit;
				string strUpperErrorLimit;

				m_AlarmDetailsTable.BeginLoadData();

				foreach(XmlNode node in AlarmNodeList)
				{
				
					ChannelRepGUID = node["ChannelRepGUID"].InnerText;
					ChannelCode = node["ChannelCode"].InnerText;
					ToolPath = node["ToolPath"].InnerText;
					//LapGUID = node["LapGUID"].InnerText;
					LapGUID = string.Empty;
				
					// Get Limits
					strLowerWarningLimit = node["LowerWarningLimit"].InnerText;
					strUpperWarningLimit = node["UpperWarningLimit"].InnerText;
					strLowerErrorLimit = node["LowerErrorLimit"].InnerText;
					strUpperErrorLimit = node["UpperErrorLimit"].InnerText;


					if(node["TimeDelay"].InnerText.Length <= 0)
					{
						DelayType = "Depth";
						if(node["DepthDelay"].InnerText.Length <= 0)
							Delay = 0;
						else
							Delay = int.Parse(node["DepthDelay"].InnerText);
					}

					if(node["DepthDelay"].InnerText.Length <= 0)
					{
						DelayType = "Time";
						if(node["TimeDelay"].InnerText.Length <= 0)
							Delay = 0;
						else
							Delay = int.Parse(node["TimeDelay"].InnerText);
					}

		
					
					if(node["AlarmOn"].InnerText.ToString().Equals("1"))
						AlarmOn = true;
					else
						AlarmOn = false;

					if(node["BeepOn"].InnerText.ToString().Equals("1"))
						BeepOn = true;
					else
						BeepOn = false;

					if(node["DisplayGuage"].InnerText.ToString().Equals("1"))
						DisplayGuage = true;
					else
						DisplayGuage = false;

					m_RepCollection.LockData();
					
					// Get representation Index for given channel representation guid.
					int repIndex = m_RepCollection.get_Index(ChannelRepGUID);
					
					// Get unti quantity and unit.
					GetUnitDetails(repIndex, "Canadian");

					m_RepCollection.UnlockData();
					
					dtRow = m_AlarmDetailsTable.NewRow();
					dtRow[DeleteColIndex] = false;
					dtRow[ChannelCodeColIndex] = ChannelCode;
					dtRow[UnitColIndex] = m_Unit;
					dtRow[AlarmOnColIndex] = AlarmOn;
					dtRow[BeepOnColIndex] = BeepOn;
					dtRow[DisplayGuageColIndex] = DisplayGuage;
					dtRow[DelayTypeColIndex] = DelayType;
					dtRow[DelayColIndex] = Delay;
					dtRow[ToolPathColIndex] = ToolPath;
					dtRow[ChannelRepGUIDColIndex] = ChannelRepGUID;
					dtRow[UnitQuantityColIndex] = m_UnitQuantity;
					dtRow[LapGUIDColIndex] = LapGUID;
					dtRow[IsAddedColIndex] = false;
					dtRow[IsModifiedColIndex] = false;

					// Limits
					if(strLowerWarningLimit.Length > 0)
						dtRow[LowerWarningLimitColIndex] = double.Parse(strLowerWarningLimit);

					if(strUpperWarningLimit.Length > 0)
						dtRow[UpperWarningLimitColIndex] = double.Parse(strUpperWarningLimit);

					if(strLowerErrorLimit.Length > 0)
						dtRow[LowerErrorLimitColIndex] = double.Parse(strLowerErrorLimit);

					if(strUpperErrorLimit.Length > 0)
						dtRow[UpperErrorLimitColIndex] = double.Parse(strUpperErrorLimit);


					m_AlarmDetailsTable.Rows.Add(dtRow);	

				}

				m_AlarmDetailsTable.EndLoadData();

			
				gridAlarmDetails.Binder.ResumeBinding();
				gridAlarmDetails.Model.ResumeChangeEvents();
				gridAlarmDetails.Model.ResumeRecordUndo();
				gridAlarmDetails.Model.ResetVolatileData();
				gridAlarmDetails.EndUpdate(true);

				// Refresh grid to reflect above changes.
				gridAlarmDetails.Refresh();	
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in PopulateAlarmGrid()");
			}

		}


		/// <summary>
		/// Convert and set the value to new unit for all channel having given unit quantity.
		/// </summary>
		private double GetValueInCanadianUS(double OldValue, string UnitQuantity)
		{
			string OldUnit = m_UnitServer.UnitForSystem(UnitQuantity, m_DisplayUnitSystem);
			string NewUnit = m_UnitServer.UnitForSystem(UnitQuantity, "Canadian");

			return ((Interface.ISlbConvertUnits)m_UnitServer).Convert(OldValue, OldUnit, NewUnit);
		}
		

		/// <summary>
		/// Check If Datatype is Supported.
		/// </summary>
		private bool CheckIfDataTypeSupported()
		{
			// TODO Use ConstMap to validate
			
			if(m_DataType == Interface.DataType_t.DATA_TYPE_INT32 || m_DataType == Interface.DataType_t.DATA_TYPE_INT16
				|| m_DataType == Interface.DataType_t.DATA_TYPE_IEEE64 || m_DataType == Interface.DataType_t.DATA_TYPE_IEEE32)
				return true;
			else
				return false;

		}


		/// <summary>
		/// Check If Limits are specified properly.
		/// </summary>
		private bool CheckLimits(string strLowerErrorLimit, string strLowerWarningLimit, string strUpperWarningLimit, string strUpperErrorLimit)
		{
			if(strLowerErrorLimit.Length <= 0 && strLowerWarningLimit.Length <= 0 && strUpperWarningLimit.Length <= 0 && strUpperErrorLimit.Length <= 0)
			{
				MessageBox.Show("Specify Error and/or Warning Limits", "Limits", MessageBoxButtons.OK,MessageBoxIcon.Error);
				return false;
			}

			if(strLowerErrorLimit.Length > 0 && strLowerWarningLimit.Length > 0)
			{
				if(double.Parse(strLowerErrorLimit) >= double.Parse(strLowerWarningLimit))
				{
					MessageBox.Show("Lower Warning should be greater than Lower Error", "Limits Incorrect", MessageBoxButtons.OK,MessageBoxIcon.Error);
					return false;
				}
			}

			if(strLowerWarningLimit.Length > 0 && strUpperWarningLimit.Length > 0)
			{
				if(double.Parse(strLowerWarningLimit) >= double.Parse(strUpperWarningLimit))
				{
					MessageBox.Show("Upper Warning should be greater than Lower Warning", "Limits Incorrect", MessageBoxButtons.OK,MessageBoxIcon.Error);
					return false;
				}
			}

			if(strUpperWarningLimit.Length > 0 && strUpperErrorLimit.Length > 0)
			{
				if(double.Parse(strUpperWarningLimit) >= double.Parse(strUpperErrorLimit))
				{
					MessageBox.Show("Upper Error should be greater than Upper Warning", "Limits Incorrect", MessageBoxButtons.OK,MessageBoxIcon.Error);
					return false;
				}
			}

			if(strLowerErrorLimit.Length > 0 && strUpperWarningLimit.Length > 0)
			{
				if(double.Parse(strLowerErrorLimit) >= double.Parse(strUpperWarningLimit))
				{
					MessageBox.Show("Upper Warning should be greater than Lower Error", "Limits Incorrect", MessageBoxButtons.OK,MessageBoxIcon.Error);
					return false;
				}
			}

			if(strLowerErrorLimit.Length > 0 && strUpperErrorLimit.Length > 0)
			{
				if(double.Parse(strLowerErrorLimit) >= double.Parse(strUpperErrorLimit))
				{
					MessageBox.Show("Upper Error should be greater than Lower Error", "Limits Incorrect", MessageBoxButtons.OK,MessageBoxIcon.Error);
					return false;
				}
			}

			if(strLowerWarningLimit.Length > 0 && strUpperErrorLimit.Length > 0)
			{
				if(double.Parse(strLowerWarningLimit) >= double.Parse(strUpperErrorLimit))
				{
					MessageBox.Show("Upper Error should be greater than Lower Warning", "Limits Incorrect", MessageBoxButtons.OK,MessageBoxIcon.Error);
					return false;
				}
			}

			return true;
																													 
		}
#endregion
		
		#region overidden methods
		/// <summary>
		/// Convert and set all the value to new unit. 
		/// </summary>
		protected override bool OnDisplayUnitSystemChanged(string OldUnitSystem, string NewUnitSystem)
		{
			string NewUnit = string.Empty;
			double Offset = 0;
			double Gain = 1;
			string NewDepthUnit = string.Empty; 
			int tempDepthDelay;

			try
			{
			
				// Check if new unit system is custom unit system.
				bool IsCustomUnitSystem = CheckIfCustomUnitSystem(NewUnitSystem);

				// To lessen flickering
				gridAlarmDetails.Model.SuspendChangeEvents();
				gridAlarmDetails.Model.SuspendRecordUndo();	
				gridAlarmDetails.BeginUpdate();
				gridAlarmDetails.Binder.SuspendBinding();

			
				// For each row in grid.
				for(int Index = 0; Index < m_AlarmDetailsTable.Rows.Count; Index++)
				{
				
					if(IsCustomUnitSystem)
					{
						// TODO
						//NewUnit = m_UnitServer.ChannelUnitForSystem(m_AlarmDetailsTable.Rows[Index][ChannelCodeColIndex], NewUnitSystem);
						NewUnit = m_UnitServer.UnitForSystem(m_AlarmDetailsTable.Rows[Index][UnitQuantityColIndex].ToString(), NewUnitSystem);
						NewDepthUnit = m_UnitServer.UnitForSystem(DepthUnitQuantity, NewUnitSystem);
					}
					else
					{
						NewUnit = m_UnitServer.UnitForSystem(m_AlarmDetailsTable.Rows[Index][UnitQuantityColIndex].ToString(), NewUnitSystem);
						NewDepthUnit = m_UnitServer.UnitForSystem(DepthUnitQuantity, NewUnitSystem);
					}
				
					// Get offset and gain to convert from one unit to other.
					//Offset = ( (Interface.ISlbConvertUnits)m_UnitServer).ConversionFactorScalar(m_AlarmDetailsTable.Rows[Index][UnitColIndex].ToString(), NewUnit, out Gain);
					Offset = ( (Interface.ISlbConvertUnits)m_UnitServer).ConversionFactorScalar(m_AlarmDetailsTable.Rows[Index][UnitColIndex].ToString(), NewUnit, out Gain);
			
					// Convert and set the values. 
					if(m_AlarmDetailsTable.Rows[Index][UpperErrorLimitColIndex].ToString().Length > 0)
						m_AlarmDetailsTable.Rows[Index][UpperErrorLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][UpperErrorLimitColIndex]) * Gain) + Offset;
					
					if(m_AlarmDetailsTable.Rows[Index][LowerErrorLimitColIndex].ToString().Length > 0)
						m_AlarmDetailsTable.Rows[Index][LowerErrorLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][LowerErrorLimitColIndex]) * Gain) + Offset;
					
					if(m_AlarmDetailsTable.Rows[Index][UpperWarningLimitColIndex].ToString().Length > 0)
						m_AlarmDetailsTable.Rows[Index][UpperWarningLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][UpperWarningLimitColIndex]) * Gain) + Offset;
					
					if(m_AlarmDetailsTable.Rows[Index][LowerWarningLimitColIndex].ToString().Length > 0)
						m_AlarmDetailsTable.Rows[Index][LowerWarningLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][LowerWarningLimitColIndex]) * Gain) + Offset;

					
					if(m_AlarmDetailsTable.Rows[Index][DelayTypeColIndex].ToString().Equals("Depth") && m_AlarmDetailsTable.Rows[Index][DelayColIndex].ToString().Length > 0)
					{
						tempDepthDelay = (int)m_AlarmDetailsTable.Rows[Index][DelayColIndex];
						tempDepthDelay = (int)( (Interface.ISlbConvertUnits)m_UnitServer).Convert(tempDepthDelay, m_DepthUnit, NewDepthUnit);
						m_AlarmDetailsTable.Rows[Index][DelayColIndex] = tempDepthDelay;
					}

					m_AlarmDetailsTable.Rows[Index][UnitColIndex] = NewUnit;
				}

				m_DisplayUnitSystem = NewUnitSystem;
				m_DepthUnit = NewDepthUnit;

				gridAlarmDetails.Binder.ResumeBinding();
				gridAlarmDetails.Model.ResumeChangeEvents();
				gridAlarmDetails.Model.ResumeRecordUndo();
				gridAlarmDetails.Model.ResetVolatileData();
				gridAlarmDetails.EndUpdate(true);

				gridAlarmDetails.Refresh();

				return true;
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in OnDisplayUnitSystemChange()");
				return false;
			}
		}


		/// <summary>
		/// Convert and set the value to new unit for all channel having given unit quantity.
		/// </summary>
		protected override void OnUnitQuantityUnitChanged(string UnitQuantity, string NewUnit, string OldUnit)
		{
			double Offset = 0;
			double Gain = 1;

			try
			{

				// For each row in grid.
				for(int Index = 0; Index < m_AlarmDetailsTable.Rows.Count; Index++)
				{
					// If unit quantity is given unit quantity
					if(UnitQuantity.Equals(m_AlarmDetailsTable.Rows[Index][UnitQuantityColIndex]))
					{
						// Get Offset and gain. 
						Offset = ( (Interface.ISlbConvertUnits)m_UnitServer).ConversionFactorScalar(m_AlarmDetailsTable.Rows[Index][UnitColIndex].ToString(), NewUnit, out Gain);

						m_AlarmDetailsTable.Rows[Index][UpperErrorLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][UpperErrorLimitColIndex]) * Gain) + Offset;
						m_AlarmDetailsTable.Rows[Index][LowerErrorLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][LowerErrorLimitColIndex]) * Gain) + Offset;
						m_AlarmDetailsTable.Rows[Index][UpperWarningLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][UpperWarningLimitColIndex]) * Gain) + Offset;
						m_AlarmDetailsTable.Rows[Index][LowerWarningLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][LowerWarningLimitColIndex]) * Gain) + Offset;

						m_AlarmDetailsTable.Rows[Index][UnitColIndex] = NewUnit;
					}
				}
				gridAlarmDetails.Refresh();
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in OnUnitQuantityChange()");
			}
		}


		/// <summary>
		/// Convert the value to new unit. 
		/// </summary>
		protected override void OnChannelDisplayUnitChanged(string ChannelCode, string NewUnit, string OldUnit)
		{
			double Offset = 0;
			double Gain = 1;
			try
			{
				// For each row in grid
				for(int Index = 0; Index < m_AlarmDetailsTable.Rows.Count; Index++)
				{
					if(ChannelCode.Equals(m_AlarmDetailsTable.Rows[Index][ChannelCodeColIndex]))
					{
						// Get Offset and gain for converting from old to new unit.
						Offset = ( (Interface.ISlbConvertUnits)m_UnitServer).ConversionFactorScalar(m_AlarmDetailsTable.Rows[Index][UnitColIndex].ToString(), NewUnit, out Gain);

						// To the conversion and set the values.
						m_AlarmDetailsTable.Rows[Index][UpperErrorLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][UpperErrorLimitColIndex]) * Gain) + Offset;
						m_AlarmDetailsTable.Rows[Index][LowerErrorLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][LowerErrorLimitColIndex]) * Gain) + Offset;
						m_AlarmDetailsTable.Rows[Index][UpperWarningLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][UpperWarningLimitColIndex]) * Gain) + Offset;
						m_AlarmDetailsTable.Rows[Index][LowerWarningLimitColIndex] = ((double)(m_AlarmDetailsTable.Rows[Index][LowerWarningLimitColIndex]) * Gain) + Offset;

						// Change the unit for Alarm.
						m_AlarmDetailsTable.Rows[Index][UnitColIndex] = NewUnit;
					}
				}
				gridAlarmDetails.Refresh();
			}
			catch (Exception exp)
			{
				MessageBox.Show(exp.Message + " in OnChannelUnitChange()");
			}
		}

		
		#endregion

	}
}
